@@ -15,17 +15,27 @@ mod x86_64_unknown_linux_gnu;
 mod x86_64_unknown_linux_musl;
 
 use crate::{
-    cargo_env, effective_target, emit_warning, env_var_to_bool, execute_command, get_crate_cflags,
-    is_no_asm, optional_env_optional_crate_target, out_dir, requested_c_std, set_env_for_target,
-    target, target_arch, target_env, target_os, CStdRequested, OutputLibType,
+    cargo_env, disable_jitter_entropy, effective_target, emit_warning, env_var_to_bool,
+    execute_command, get_crate_cflags, is_no_asm, optional_env_optional_crate_target,
+    optional_env_target, out_dir, requested_c_std, set_env_for_target, target, target_arch,
+    target_env, target_os, CStdRequested, OutputLibType,
 };
+use std::cell::Cell;
+use std::collections::HashMap;
 use std::path::PathBuf;
 
+#[non_exhaustive]
+#[derive(PartialEq, Eq)]
+pub(crate) enum CompilerFeature {
+    NeonSha3,
+}
+
 pub(crate) struct CcBuilder {
     manifest_dir: PathBuf,
     out_dir: PathBuf,
     build_prefix: Option<String>,
     output_lib_type: OutputLibType,
+    compiler_features: Cell<Vec<CompilerFeature>>,
 }
 
 use std::fs;
@@ -168,6 +178,7 @@ impl CcBuilder {
             out_dir,
             build_prefix,
             output_lib_type,
+            compiler_features: Cell::new(vec![]),
         }
     }
 
@@ -190,7 +201,7 @@ impl CcBuilder {
                 build_options.push(BuildOption::std("c11"));
             }
             CStdRequested::None => {
-                if self.compiler_check("c11") {
+                if self.compiler_check("c11", Vec::<String>::new()) {
                     build_options.push(BuildOption::std("c11"));
                 } else {
                     build_options.push(BuildOption::std("c99"));
@@ -276,7 +287,9 @@ impl CcBuilder {
                 build_options.push(BuildOption::flag("-pthread"));
             }
         }
-
+        if Some(true) == disable_jitter_entropy() {
+            build_options.push(BuildOption::define("DISABLE_CPU_JITTER_ENTROPY", "1"));
+        }
         self.add_includes(&mut build_options);
 
         build_options
@@ -310,13 +323,16 @@ impl CcBuilder {
                 .join("s2n-bignum-imported")
                 .join("include"),
         ));
-        build_options.push(BuildOption::include(
-            self.manifest_dir
-                .join("aws-lc")
-                .join("third_party")
-                .join("jitterentropy")
-                .join("jitterentropy-library"),
-        ));
+
+        if Some(true) != disable_jitter_entropy() {
+            build_options.push(BuildOption::include(
+                self.manifest_dir
+                    .join("aws-lc")
+                    .join("third_party")
+                    .join("jitterentropy")
+                    .join("jitterentropy-library"),
+            ));
+        }
     }
 
     pub fn create_builder(&self) -> cc::Build {
@@ -341,13 +357,80 @@ impl CcBuilder {
         cc_build
     }
 
+    #[allow(clippy::zero_sized_map_values)]
+    fn build_s2n_bignum_source_feature_map() -> HashMap<String, CompilerFeature> {
+        let mut source_feature_map: HashMap<String, CompilerFeature> = HashMap::new();
+        source_feature_map.insert("sha3_keccak_f1600_alt.S".into(), CompilerFeature::NeonSha3);
+        source_feature_map.insert("sha3_keccak2_f1600.S".into(), CompilerFeature::NeonSha3);
+        source_feature_map.insert(
+            "sha3_keccak4_f1600_alt2.S".into(),
+            CompilerFeature::NeonSha3,
+        );
+        source_feature_map
+    }
+
+    fn prepare_jitter_entropy_builder(&self) -> cc::Build {
+        // See: https://github.com/aws/aws-lc/blob/2294510cd0ecb2d5946461e3dbb038363b7b94cb/third_party/jitterentropy/CMakeLists.txt#L19-L35
+        let mut build_options: Vec<BuildOption> = Vec::new();
+        self.add_includes(&mut build_options);
+
+        let mut je_builder = cc::Build::new();
+        for option in build_options {
+            option.apply_cc(&mut je_builder);
+        }
+
+        let compiler = if let Some(original_cflags) = optional_env_target("CFLAGS") {
+            let mut new_cflags = original_cflags.clone();
+            new_cflags.push_str(" -O0");
+            set_env_for_target("CFLAGS", &new_cflags);
+            // cc-rs currently prioritizes flags provided by CFLAGS over the flags provided by the build script.
+            // The environment variables used by the compiler are set when `get_compiler` is called.
+            let compiler = je_builder.get_compiler();
+            set_env_for_target("CFLAGS", &original_cflags);
+            compiler
+        } else {
+            je_builder.get_compiler()
+        };
+
+        je_builder.define("AWSLC", "1");
+        je_builder.pic(true);
+        if target_os() == "windows" && compiler.is_like_msvc() {
+            je_builder.flag("/Od").flag("/W4").flag("/DYNAMICBASE");
+        } else {
+            je_builder
+                .flag("-fwrapv")
+                .flag("--param")
+                .flag("ssp-buffer-size=4")
+                .flag("-fvisibility=hidden")
+                .flag("-Wcast-align")
+                .flag("-Wmissing-field-initializers")
+                .flag("-Wshadow")
+                .flag("-Wswitch-enum")
+                .flag("-Wextra")
+                .flag("-Wall")
+                .flag("-pedantic")
+                // Compilation will fail if optimizations are enabled.
+                .flag("-O0")
+                .flag("-fwrapv")
+                .flag("-Wconversion");
+        }
+        je_builder
+    }
+
     fn add_all_files(&self, lib: &Library, cc_build: &mut cc::Build) {
         use core::str::FromStr;
+        let compiler = cc_build.get_compiler();
 
-        // s2n_bignum is compiled separately due to needing extra flags
+        let force_include_option = if compiler.is_like_msvc() {
+            "/FI"
+        } else {
+            "--include="
+        };
+        // s2n-bignum is compiled separately due to needing extra flags
         let mut s2n_bignum_builder = cc_build.clone();
         s2n_bignum_builder.flag(format!(
-            "--include={}",
+            "{}{}",
+            force_include_option,
             self.manifest_dir
                 .join("generated-include")
                 .join("openssl")
@@ -355,20 +438,71 @@ impl CcBuilder {
                 .display()
         ));
         s2n_bignum_builder.define("S2N_BN_HIDE_SYMBOLS", "1");
+
+        // CPU Jitter Entropy is compiled separately due to needing specific flags
+        let mut jitter_entropy_builder = self.prepare_jitter_entropy_builder();
+        jitter_entropy_builder.flag(format!(
+            "{}{}",
+            force_include_option,
+            self.manifest_dir
+                .join("generated-include")
+                .join("openssl")
+                .join("boringssl_prefix_symbols.h")
+                .display()
+        ));
+
+        let s2n_bignum_source_feature_map = Self::build_s2n_bignum_source_feature_map();
+        let compiler_features = self.compiler_features.take();
         for source in lib.sources {
             let source_path = self.manifest_dir.join("aws-lc").join(source);
             let is_s2n_bignum = std::path::Path::new(source).starts_with("third_party/s2n-bignum");
+            let is_jitter_entropy =
+                std::path::Path::new(source).starts_with("third_party/jitterentropy");
 
+            if !source_path.is_file() {
+                emit_warning(&format!("Not a file: {:?}", source_path.as_os_str()));
+                continue;
+            }
             if is_s2n_bignum {
-                s2n_bignum_builder.file(source_path);
+                let filename: String = source_path
+                    .file_name()
+                    .unwrap()
+                    .to_str()
+                    .unwrap()
+                    .to_string();
+
+                if let Some(compiler_feature) = s2n_bignum_source_feature_map.get(&filename) {
+                    if compiler_features.contains(compiler_feature) {
+                        s2n_bignum_builder.file(source_path);
+                    } else {
+                        emit_warning(&format!(
+                            "Skipping due to missing compiler features: {:?}",
+                            source_path.as_os_str()
+                        ));
+                    }
+                } else {
+                    s2n_bignum_builder.file(source_path);
+                }
+            } else if is_jitter_entropy {
+                // Only compile if not disabled.
+                if Some(true) != disable_jitter_entropy() {
+                    jitter_entropy_builder.file(source_path);
+                }
             } else {
                 cc_build.file(source_path);
             }
         }
-        let object_files = s2n_bignum_builder.compile_intermediates();
-        for object in object_files {
+        self.compiler_features.set(compiler_features);
+        let s2n_bignum_object_files = s2n_bignum_builder.compile_intermediates();
+        for object in s2n_bignum_object_files {
             cc_build.object(object);
         }
+        if Some(true) != disable_jitter_entropy() {
+            let jitter_entropy_object_files = jitter_entropy_builder.compile_intermediates();
+            for object in jitter_entropy_object_files {
+                cc_build.object(object);
+            }
+        }
         cc_build.file(PathBuf::from_str("rust_wrapper.c").unwrap());
     }
 
@@ -390,7 +524,11 @@ impl CcBuilder {
     // This performs basic checks of compiler capabilities and sets an appropriate flag on success.
     // This should be kept in alignment with the checks performed by AWS-LC's CMake build.
     // See: https://github.com/search?q=repo%3Aaws%2Faws-lc%20check_compiler&type=code
-    fn compiler_check(&self, basename: &str) -> bool {
+    fn compiler_check<T, S>(&self, basename: &str, extra_flags: T) -> bool
+    where
+        T: IntoIterator<Item = S>,
+        S: AsRef<str>,
+    {
         let mut ret_val = false;
         let output_dir = self.out_dir.join(format!("out-{basename}"));
         let source_file = self
@@ -415,6 +553,10 @@ impl CcBuilder {
             .file(source_file)
             .warnings_into_errors(true)
             .out_dir(&output_dir);
+        for flag in extra_flags {
+            let flag = flag.as_ref();
+            cc_build.flag(flag);
+        }
 
         let compiler = cc_build.get_compiler();
         if compiler.is_like_gnu() || compiler.is_like_clang() {
@@ -505,12 +647,28 @@ impl CcBuilder {
         let _ = fs::remove_file(exec_path);
     }
     fn run_compiler_checks(&self, cc_build: &mut cc::Build) {
-        if self.compiler_check("stdalign_check") {
+        if self.compiler_check("stdalign_check", Vec::<&'static str>::new()) {
             cc_build.define("AWS_LC_STDALIGN_AVAILABLE", Some("1"));
         }
-        if self.compiler_check("builtin_swap_check") {
+        if self.compiler_check("builtin_swap_check", Vec::<&'static str>::new()) {
             cc_build.define("AWS_LC_BUILTIN_SWAP_SUPPORTED", Some("1"));
         }
+        if target_arch() == "aarch64"
+            && self.compiler_check("neon_sha3_check", vec!["-march=armv8.4-a+sha3"])
+        {
+            let mut compiler_features = self.compiler_features.take();
+            compiler_features.push(CompilerFeature::NeonSha3);
+            self.compiler_features.set(compiler_features);
+            cc_build.define("MY_ASSEMBLER_SUPPORTS_NEON_SHA3_EXTENSION", Some("1"));
+        }
+        if target_os() == "linux" {
+            if self.compiler_check("linux_random_h", Vec::<&'static str>::new()) {
+                cc_build.define("HAVE_LINUX_RANDOM_H", Some("1"));
+            } else if self.compiler_check("linux_random_h", vec!["-DDEFINE_U32"]) {
+                cc_build.define("HAVE_LINUX_RANDOM_H", Some("1"));
+                cc_build.define("AWS_LC_URANDOM_NEEDS_U32", Some("1"));
+            }
+        }
         self.memcmp_check();
     }
 }
