@@ -4,7 +4,7 @@
  */
 
 /* Automatically managed default lints */
-#![cfg_attr(docsrs, feature(doc_auto_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 /* End of automatically managed default lints */
 #![allow(clippy::derive_partial_eq_without_eq)]
 #![warn(
@@ -21,6 +21,7 @@ pub mod arbitrary;
 mod buf;
 pub mod error;
 pub mod frame;
+pub mod message_size_hint;
 pub mod smithy;
 #[cfg(feature = "test-util")]
 pub mod test_util;
