@@ -115,7 +115,7 @@ pub trait ImgExt<Pixel> {
     fn rows_padded(&self) -> slice::Chunks<'_, Pixel>;
 
     /// Borrow the container
-    fn as_ref(&self) -> ImgRef<Pixel>;
+    fn as_ref(&self) -> ImgRef<'_, Pixel>;
 }
 
 /// Additional methods that depend on buffer size
@@ -133,7 +133,7 @@ pub trait ImgExtMut<Pixel> {
     fn rows_padded_mut(&mut self) -> slice::ChunksMut<'_, Pixel>;
 
     /// Borrow the container mutably
-    fn as_mut(&mut self) -> ImgRefMut<Pixel>;
+    fn as_mut(&mut self) -> ImgRefMut<'_, Pixel>;
 }
 
 /// Basic struct used for both owned (alias `ImgVec`) and borrowed (alias `ImgRef`) image fragments.
@@ -268,7 +268,7 @@ impl<Pixel,Container> ImgExt<Pixel> for Img<Container> where Container: AsRef<[P
 
     #[inline(always)]
     #[allow(deprecated)]
-    fn as_ref(&self) -> ImgRef<Pixel> {
+    fn as_ref(&self) -> ImgRef<'_, Pixel> {
         Img {
             buf: self.buf.as_ref(),
             width: self.width,
@@ -286,8 +286,6 @@ impl<Pixel,Container> ImgExtMut<Pixel> for Img<Container> where Container: AsMut
     /// # Panics
     ///
     /// If stride is 0
-    #[inline]
-    #[must_use]
     #[cfg(feature = "deprecated")]
     fn rows_padded_mut(&mut self) -> slice::ChunksMut<'_, Pixel> {
         let stride = self.stride();
@@ -296,7 +294,7 @@ impl<Pixel,Container> ImgExtMut<Pixel> for Img<Container> where Container: AsMut
 
     #[inline(always)]
     #[allow(deprecated)]
-    fn as_mut(&mut self) -> ImgRefMut<Pixel> {
+    fn as_mut(&mut self) -> ImgRefMut<'_, Pixel> {
         Img {
             buf: self.buf.as_mut(),
             width: self.width,
@@ -328,7 +326,7 @@ impl<'slice, T> ImgRef<'slice, T> {
     ///
     /// # Panics
     ///
-    /// It will panic if sub_image is outside of the image area
+    /// It will panic if `sub_image` is outside of the image area
     /// (left + width must be <= container width, etc.)
     #[inline]
     #[must_use]
@@ -396,9 +394,14 @@ impl<'slice, T> ImgRefMut<'slice, T> {
         self.as_ref().sub_image(left, top, width, height)
     }
 
-    /// Trim this image without copying.
+    /// Slices this image reference to produce another reference to a subregion of it.
+    ///
     /// Note that mutable borrows are exclusive, so it's not possible to have more than
     /// one mutable subimage at a time.
+    ///
+    /// ## Panics
+    ///
+    /// If the coordinates are out of bounds
     #[inline]
     #[allow(deprecated)]
     #[must_use]
@@ -411,6 +414,25 @@ impl<'slice, T> ImgRefMut<'slice, T> {
         ImgRefMut::new_stride(buf, width, height, stride)
     }
 
+    /// Transforms this image reference to refer to a subregion.
+    /// 
+    /// This is identical in behavior to [`ImgRefMut::sub_image_mut()`], except that it returns an
+    /// [`ImgRefMut`] with the same lifetime, rather than a reborrow with a shorter lifetime.
+    ///
+    /// ## Panics
+    ///
+    /// If the coordinates are out of bounds
+    #[allow(deprecated)]
+    #[must_use]
+    #[track_caller]
+    pub fn into_sub_image_mut(self, left: usize, top: usize, width: usize, height: usize) -> Self {
+        assert!(top + height <= self.height());
+        assert!(left + width <= self.width());
+        let (start, end, stride) = sub_image(left, top, width, height, self.stride(), self.buf.len());
+        let buf = &mut self.buf[start..end];
+        ImgRefMut::new_stride(buf, width, height, stride)
+    }
+
     /// Make mutable reference immutable
     #[inline]
     #[must_use]
@@ -458,7 +480,9 @@ impl<T: Copy> ImgRefMut<'_, T> {
     pub fn pixels(&self) -> PixelsIter<'_, T> {
         PixelsIter::new(self.as_ref())
     }
+}
 
+impl<T> ImgRefMut<'_, T> {
     /// If you want to iterate in parallel, parallelize `rows()` instead.
     /// # Panics
     ///
@@ -478,7 +502,9 @@ impl<T: Copy> ImgVec<T> {
     pub fn pixels(&self) -> PixelsIter<'_, T> {
         PixelsIter::new(self.as_ref())
     }
+}
 
+impl<T> ImgVec<T> {
     /// If you want to iterate in parallel, parallelize `rows()` instead.
     /// # Panics
     ///
@@ -532,6 +558,10 @@ impl<Container> IntoIterator for Img<Container> where Container: IntoIterator {
 
 impl<T> ImgVec<T> {
     /// Create a mutable view into a region within the image. See `sub_image()` for read-only views.
+    ///
+    /// ## Panics
+    ///
+    /// If the coordinates are out of bounds
     #[allow(deprecated)]
     #[must_use]
     #[track_caller]
@@ -617,6 +647,10 @@ impl<Container> Img<Container> {
     /// Stride can be equal to `width` or larger. If it's larger, then pixels between end of previous row and start of the next are considered a padding, and may be ignored.
     ///
     /// The `Container` is usually a `Vec` or a slice.
+    ///
+    /// ## Panics
+    ///
+    /// If stride is 0.
     #[inline]
     #[allow(deprecated)]
     #[track_caller]
@@ -871,13 +905,21 @@ mod tests {
         assert_eq!(1, subimg.rows().count());
         }
 
-        let mut img = img;
-        let mut subimg = img.sub_image_mut(1, 1, 2, 1);
-        assert_eq!(1, subimg.rows().count());
-        assert_eq!(1, subimg.rows_mut().count());
-        assert_eq!(1, subimg.rows_mut().rev().count());
-        assert_eq!(1, subimg.rows_mut().fuse().rev().count());
-        assert_eq!(subimg.buf()[0], 6);
+        // 3 different methods for constructing mutable sub-images
+        let mut img_for_mut_1 = img.clone();
+        let mut img_for_mut_2 = img.clone();
+        let mut img_for_mut_3 = img;
+        for mut subimg in [
+            img_for_mut_1.sub_image_mut(1, 1, 2, 1),
+            img_for_mut_2.as_mut().sub_image_mut(1, 1, 2, 1),
+            img_for_mut_3.as_mut().into_sub_image_mut(1, 1, 2, 1),
+        ] {
+            assert_eq!(1, subimg.rows().count());
+            assert_eq!(1, subimg.rows_mut().count());
+            assert_eq!(1, subimg.rows_mut().rev().count());
+            assert_eq!(1, subimg.rows_mut().fuse().rev().count());
+            assert_eq!(subimg.buf()[0], 6);
+        }
     }
 
     #[test]
