@@ -2,15 +2,14 @@
 
 use core::ops::{Index, IndexMut};
 
-use ttf_parser::opentype_layout::{FeatureIndex, LanguageIndex, LookupIndex, ScriptIndex};
-use ttf_parser::GlyphId;
-
 use super::buffer::*;
 use super::common::TagExt;
 use super::ot_layout_gsubgpos::{Apply, OT};
 use super::ot_shape_plan::hb_ot_shape_plan_t;
 use super::unicode::{hb_unicode_funcs_t, hb_unicode_general_category_t, GeneralCategoryExt};
 use super::{hb_font_t, hb_glyph_info_t, hb_tag_t};
+use crate::hb::set_digest::{hb_set_digest_ext, hb_set_digest_t};
+use ttf_parser::opentype_layout::{FeatureIndex, LanguageIndex, LookupIndex, ScriptIndex};
 
 pub const MAX_NESTING_LEVEL: usize = 64;
 pub const MAX_CONTEXT_LENGTH: usize = 64;
@@ -42,7 +41,6 @@ pub fn _hb_ot_layout_set_glyph_props(face: &hb_font_t, buffer: &mut hb_buffer_t)
     for info in &mut buffer.info[..len] {
         info.set_glyph_props(face.glyph_props(info.as_glyph()));
         info.set_lig_props(0);
-        info.set_syllable(0);
     }
 }
 
@@ -103,8 +101,8 @@ pub trait LayoutLookup: Apply {
     /// Whether the lookup has to be applied backwards.
     fn is_reverse(&self) -> bool;
 
-    /// Whether any subtable of the lookup could apply at a specific glyph.
-    fn covers(&self, glyph: GlyphId) -> bool;
+    /// The digest of the lookup.
+    fn digest(&self) -> &hb_set_digest_t;
 }
 
 pub trait LayoutTableExt {
@@ -240,30 +238,36 @@ pub fn apply_layout_table<T: LayoutTable>(
     let mut ctx = OT::hb_ot_apply_context_t::new(T::INDEX, face, buffer);
 
     for (stage_index, stage) in plan.ot_map.stages(T::INDEX).iter().enumerate() {
-        for lookup in plan.ot_map.stage_lookups(T::INDEX, stage_index) {
-            ctx.lookup_index = lookup.index;
-            ctx.lookup_mask = lookup.mask;
-            ctx.auto_zwj = lookup.auto_zwj;
-            ctx.auto_zwnj = lookup.auto_zwnj;
+        if let Some(table) = table {
+            for lookup_map in plan.ot_map.stage_lookups(T::INDEX, stage_index) {
+                let Some(lookup) = table.get_lookup(lookup_map.index) else {
+                    continue;
+                };
 
-            ctx.random = lookup.random;
-            ctx.per_syllable = lookup.per_syllable;
+                if lookup.digest().may_have(&ctx.digest) {
+                    ctx.lookup_index = lookup_map.index;
+                    ctx.set_lookup_mask(lookup_map.mask);
+                    ctx.auto_zwj = lookup_map.auto_zwj;
+                    ctx.auto_zwnj = lookup_map.auto_zwnj;
+
+                    ctx.random = lookup_map.random;
+                    ctx.per_syllable = lookup_map.per_syllable;
 
-            if let Some(table) = &table {
-                if let Some(lookup) = table.get_lookup(lookup.index) {
                     apply_string::<T>(&mut ctx, lookup);
                 }
             }
         }
 
         if let Some(func) = stage.pause_func {
-            func(plan, face, ctx.buffer);
+            if func(plan, face, ctx.buffer) {
+                ctx.digest = ctx.buffer.digest();
+            }
         }
     }
 }
 
 fn apply_string<T: LayoutTable>(ctx: &mut OT::hb_ot_apply_context_t, lookup: &T::Lookup) {
-    if ctx.buffer.is_empty() || ctx.lookup_mask == 0 {
+    if ctx.buffer.is_empty() || ctx.lookup_mask() == 0 {
         return;
     }
 
@@ -293,7 +297,7 @@ fn apply_forward(ctx: &mut OT::hb_ot_apply_context_t, lookup: &impl Apply) -> bo
     let mut ret = false;
     while ctx.buffer.idx < ctx.buffer.len && ctx.buffer.successful {
         let cur = ctx.buffer.cur(0);
-        if (cur.mask & ctx.lookup_mask) != 0
+        if (cur.mask & ctx.lookup_mask()) != 0
             && ctx.check_glyph_property(cur, ctx.lookup_props)
             && lookup.apply(ctx).is_some()
         {
@@ -309,7 +313,7 @@ fn apply_backward(ctx: &mut OT::hb_ot_apply_context_t, lookup: &impl Apply) -> b
     let mut ret = false;
     loop {
         let cur = ctx.buffer.cur(0);
-        ret |= (cur.mask & ctx.lookup_mask) != 0
+        ret |= (cur.mask & ctx.lookup_mask()) != 0
             && ctx.check_glyph_property(cur, ctx.lookup_props)
             && lookup.apply(ctx).is_some();
 
@@ -326,12 +330,12 @@ fn apply_backward(ctx: &mut OT::hb_ot_apply_context_t, lookup: &impl Apply) -> b
 
 /* Design:
  * unicode_props() is a two-byte number.  The low byte includes:
- * - General_Category: 5 bits.
+ * - Modified General_Category: 5 bits.
  * - A bit each for:
  *   * Is it Default_Ignorable(); we have a modified Default_Ignorable().
  *   * Whether it's one of the four Mongolian Free Variation Selectors,
  *     CGJ, or other characters that are hidden but should not be ignored
- *     like most other Default_Ignorable()s do during matching.
+ *     like most other Default_Ignorable()s do during GSUB matching.
  *   * Whether it's a grapheme continuation.
  *
  * The high-byte has different meanings, switched by the Gen-Cat:
@@ -339,6 +343,11 @@ fn apply_backward(ctx: &mut OT::hb_ot_apply_context_t, lookup: &impl Apply) -> b
  * - For Cf: whether it's ZWJ, ZWNJ, or something else.
  * - For Ws: index of which space character this is, if space fallback
  *   is needed, ie. we don't set this by default, only if asked to.
+ *
+ * Above I said "modified" General_Category. This is because we need to
+ * remember Variation Selectors, and we don't have bits left. So we
+ * change their Gen_Cat from Mn to Cf, and use a bit of the high byte to
+ * remember them.
  */
 
 //  enum hb_unicode_props_flags_t {
@@ -483,19 +492,41 @@ pub(crate) fn _hb_glyph_info_get_unicode_space_fallback_type(
     }
 }
 
+#[inline]
+pub(crate) fn _hb_glyph_info_is_variation_selector(info: &hb_glyph_info_t) -> bool {
+    let a = _hb_glyph_info_get_general_category(info) == hb_unicode_general_category_t::Format;
+    let b = (info.unicode_props() & UnicodeProps::CF_VS.bits()) != 0;
+    a && b
+}
+
+#[inline]
+pub(crate) fn _hb_glyph_info_set_variation_selector(info: &mut hb_glyph_info_t, customize: bool) {
+    if customize {
+        _hb_glyph_info_set_general_category(info, hb_unicode_general_category_t::Format);
+        info.set_unicode_props(info.unicode_props() | UnicodeProps::CF_VS.bits())
+    } else {
+        // Reset to their original condition
+        _hb_glyph_info_set_general_category(info, hb_unicode_general_category_t::NonspacingMark);
+    }
+}
+
 #[inline]
 pub(crate) fn _hb_glyph_info_is_default_ignorable(info: &hb_glyph_info_t) -> bool {
     let n = info.unicode_props() & UnicodeProps::IGNORABLE.bits();
     n != 0 && !_hb_glyph_info_substituted(info)
 }
 
-//   static inline bool
-//   _hb_glyph_info_is_default_ignorable_and_not_hidden (const hb_glyph_info_t *info)
-//   {
-//     return ((info->unicode_props() & (UPROPS_MASK_IGNORABLE|UPROPS_MASK_HIDDEN))
-//         == UPROPS_MASK_IGNORABLE) &&
-//        !_hb_glyph_info_substituted (info);
-//   }
+#[inline]
+pub(crate) fn _hb_glyph_info_clear_default_ignorable(info: &mut hb_glyph_info_t) {
+    let mut n = info.unicode_props();
+    n &= !UnicodeProps::IGNORABLE.bits();
+    info.set_unicode_props(n);
+}
+
+#[inline]
+pub(crate) fn _hb_glyph_info_is_hidden(info: &hb_glyph_info_t) -> bool {
+    (info.unicode_props() & UnicodeProps::HIDDEN.bits()) != 0
+}
 
 //   static inline void
 //   _hb_glyph_info_unhide (hb_glyph_info_t *info)
@@ -715,9 +746,11 @@ pub fn _hb_clear_substitution_flags(
     _: &hb_ot_shape_plan_t,
     _: &hb_font_t,
     buffer: &mut hb_buffer_t,
-) {
+) -> bool {
     let len = buffer.len;
     for info in &mut buffer.info[..len] {
         _hb_glyph_info_clear_substituted(info);
     }
+
+    false
 }
