@@ -232,6 +232,10 @@ pub trait ValueType: Sized {
         Self::try_from(v).expect(msg)
     }
 
+    fn is_option() -> bool {
+        false
+    }
+
     fn type_name() -> String;
 
     fn array_type() -> ArrayType;
@@ -643,6 +647,10 @@ where
         }
     }
 
+    fn is_option() -> bool {
+        true
+    }
+
     fn type_name() -> String {
         format!("Option<{}>", T::type_name())
     }
@@ -1999,7 +2007,7 @@ mod tests {
 
         let query = Query::select().expr(timestamp).to_owned();
 
-        let formatted = "2020-01-01 02:02:02 +08:00";
+        let formatted = "2020-01-01 02:02:02.000000 +08:00";
 
         assert_eq!(
             query.to_string(MysqlQueryBuilder),
