@@ -91,6 +91,8 @@ cfg_if! {
 		#[derive(Clone)]
 		pub struct Settings {
 			pub termios: RawTermios,
+			#[cfg(target_os = "aix")]
+			pub rts_cts: bool,
 		}
 
 		impl Settings {
@@ -98,11 +100,19 @@ cfg_if! {
 				unsafe {
 					let mut termios = std::mem::zeroed();
 					check(libc::tcgetattr(file.as_raw_fd(), &mut termios))?;
-					Ok(Settings { termios })
+					Ok(Settings {
+						termios,
+						#[cfg(target_os = "aix")]
+						rts_cts: false,
+					})
 				}
 			}
 
 			fn set_on_file(&self, file: &mut std::fs::File) -> std::io::Result<()> {
+				#[cfg(target_os = "aix")]
+				if self.rts_cts {
+					return Err(std::io::Error::other("RTS/CTS flow control is not supported on AIX"));
+				}
 				unsafe {
 					check(libc::tcsetattr(file.as_raw_fd(), libc::TCSADRAIN, &self.termios))?;
 					Ok(())
@@ -293,11 +303,12 @@ impl SerialPort {
 	pub fn discard_buffers(&self, discard_input: bool, discard_output: bool) -> std::io::Result<()> {
 		unsafe {
 			let mut flags = 0;
-			if discard_input {
-				flags |= libc::TCIFLUSH;
-			}
-			if discard_output {
-				flags |= libc::TCOFLUSH;
+			if discard_input && discard_output {
+				flags = libc::TCIOFLUSH;
+			} else if discard_input {
+				flags = libc::TCIFLUSH;
+			} else if discard_output {
+				flags = libc::TCOFLUSH;
 			}
 			check(libc::tcflush(self.file.as_raw_fd(), flags))?;
 			Ok(())
@@ -395,7 +406,7 @@ fn other_error<E>(msg: E) -> std::io::Error
 where
 	E: Into<Box<dyn std::error::Error + Send + Sync>>,
 {
-	std::io::Error::new(std::io::ErrorKind::Other, msg)
+	std::io::Error::other(msg)
 }
 
 #[cfg(any(doc, feature = "doc", all(unix, feature = "unix")))]
@@ -410,6 +421,7 @@ fn pts_name(master: &SerialPort) -> std::io::Result<std::path::PathBuf> {
 				target_os = "netbsd",
 				target_os = "illumos",
 				target_os = "solaris",
+				target_os = "aix",
 		))] {
 			static PTSNAME: std::sync::Mutex<()> = std::sync::Mutex::new(());
 			let _lock = PTSNAME.lock();
@@ -513,17 +525,15 @@ impl Settings {
 					let baud_rate = libc::cfgetospeed(&self.termios);
 					#[allow(clippy::useless_conversion)] // Not useless on all platforms.
 					baud_rate.try_into()
-						.map_err(|_| std::io::Error::new(std::io::ErrorKind::Other, format!("baud rate out of range: {} > {}", baud_rate, u32::MAX)))
+						.map_err(|_| std::io::Error::other(format!("baud rate out of range: {} > {}", baud_rate, u32::MAX)))
 				}
+			} else if #[cfg(all(
+				any(target_os = "android", target_os = "linux"),
+				not(any(target_arch = "powerpc", target_arch = "powerpc64"))
+			))]
+			{
+				Ok(self.termios.c_ospeed)
 			} else {
-				#[cfg(all(
-					any(target_os = "android", target_os = "linux"),
-					not(any(target_arch = "powerpc", target_arch = "powerpc64"))
-				))]
-				if self.termios.c_cflag & libc::CBAUD == libc::BOTHER {
-					return Ok(self.termios.c_ospeed);
-				}
-
 				unsafe {
 					let termios_speed = libc::cfgetospeed(&self.termios as *const _ as *const _ );
 					let &(_, bits_per_second) = BAUD_RATES.iter()
@@ -593,15 +603,28 @@ impl Settings {
 		match flow_control {
 			crate::FlowControl::None => {
 				self.termios.c_iflag &= !(libc::IXON | libc::IXOFF);
-				self.termios.c_cflag &= !libc::CRTSCTS;
+				#[cfg(not(target_os = "aix"))]
+				{
+					self.termios.c_cflag &= !libc::CRTSCTS;
+				}
 			},
 			crate::FlowControl::XonXoff => {
 				self.termios.c_iflag |= libc::IXON | libc::IXOFF;
-				self.termios.c_cflag &= !libc::CRTSCTS;
+				#[cfg(not(target_os = "aix"))]
+				{
+					self.termios.c_cflag &= !libc::CRTSCTS;
+				}
 			},
 			crate::FlowControl::RtsCts => {
-				self.termios.c_iflag &= !(libc::IXON | libc::IXOFF);
-				self.termios.c_cflag |= libc::CRTSCTS;
+				#[cfg(target_os = "aix")]
+				{
+					self.rts_cts = true;
+				}
+				#[cfg(not(target_os = "aix"))]
+				{
+					self.termios.c_iflag &= !(libc::IXON | libc::IXOFF);
+					self.termios.c_cflag |= libc::CRTSCTS;
+				}
 			},
 		};
 	}
@@ -609,6 +632,9 @@ impl Settings {
 	pub fn get_flow_control(&self) -> std::io::Result<crate::FlowControl> {
 		let ixon = self.termios.c_iflag & libc::IXON != 0;
 		let ixoff = self.termios.c_iflag & libc::IXOFF != 0;
+		#[cfg(target_os = "aix")]
+		let crtscts = false;
+		#[cfg(not(target_os = "aix"))]
 		let crtscts = self.termios.c_cflag & libc::CRTSCTS != 0;
 
 		if !crtscts && !ixon && !ixoff {
