@@ -614,7 +614,7 @@ impl Default for Stretch {
 ///
 /// This is generated by the [`synthesize`](Attributes::synthesize) method on
 /// [`Attributes`].
-#[derive(Copy, Clone, Default)]
+#[derive(Copy, Clone, Default, Debug)]
 pub struct Synthesis {
     vars: [Setting<f32>; 4],
     len: u8,
