@@ -683,14 +683,13 @@ impl core::fmt::Debug for WER_SUBMIT_RESULT {
 }
 #[repr(transparent)]
 #[derive(Clone, Copy, Debug, PartialEq, Eq)]
-pub struct HREPORT(pub *mut core::ffi::c_void);
+pub struct HREPORT(pub isize);
 impl HREPORT {
     pub fn is_invalid(&self) -> bool {
-        self.0 == -1 as _ || self.0 == 0 as _
+        self.0 == -1 || self.0 == 0
     }
 }
 impl windows_core::Free for HREPORT {
-    #[inline]
     unsafe fn free(&mut self) {
         if !self.is_invalid() {
             _ = WerReportCloseHandle(*self);
@@ -707,14 +706,13 @@ impl windows_core::TypeKind for HREPORT {
 }
 #[repr(transparent)]
 #[derive(Clone, Copy, Debug, PartialEq, Eq)]
-pub struct HREPORTSTORE(pub *mut core::ffi::c_void);
+pub struct HREPORTSTORE(pub isize);
 impl HREPORTSTORE {
     pub fn is_invalid(&self) -> bool {
-        self.0 == -1 as _ || self.0 == 0 as _
+        self.0 == -1 || self.0 == 0
     }
 }
 impl windows_core::Free for HREPORTSTORE {
-    #[inline]
     unsafe fn free(&mut self) {
         if !self.is_invalid() {
             WerStoreClose(*self);
